import {
  BeforeApplicationShutdown,
  Injectable,
  OnApplicationShutdown,
  OnModuleInit,
  Query,
} from '@nestjs/common';
import * as config from '../config.js';
import { Bus, Control, Strip } from 'voicemeeter-remote-library/dist';
import { VoicemeeterType } from 'voicemeeter-remote-library/dist/VMR/Control';
import { OutputBuses } from 'voicemeeter-remote-library/dist/VMR/Strip.js';

export enum IOType {
  STRIP,
  BUS,
}

@Injectable()
export class AppService implements OnModuleInit {
  private control = new Control(config.dllPath, config.headerPath);
  private strips: Strip[];
  private buses: Bus[];
  async onModuleInit() {
    await this.control.initialize();
    this.control.login();
    this.strips = [...Array(this.getAvailableIOs().strips.total).keys()].map(
      (id) => new Strip(this.control, id),
    );
    this.buses = [...Array(this.getAvailableIOs().buses.total).keys()].map(
      (id) => new Bus(this.control, id),
    );
  }

  getType() {
    return {
      typeId: this.control.voicemeeterType,
      type: VoicemeeterType[this.control.voicemeeterType],
    };
  }
  getAvailableIOs() {
    return {
      buses: this.control.availableBuses,
      strips: this.control.availableStrips,
    };
  }

  getValues() {
    return {
      strips: this.strips.map((v) => ({
        gain: v.Gain,
        muteState: v.Mute,
        outputBuses: [
          ...Object.values(v.outputBuses.A),
          ...Object.values(v.outputBuses.B),
        ],
      })),
      buses: this.buses.map((v) => ({ gain: v.Gain, muteState: v.Mute })),
    };
  }

  getGain(ioType: IOType, id: number) {
    if (ioType === IOType.BUS) {
      return this.buses[id].Gain;
    } else if (ioType === IOType.STRIP) {
      return this.strips[id].Gain;
    }
  }

  setGain(ioType: IOType, id: number, gain: number) {
    if (ioType === IOType.BUS) {
      return (this.buses[id].Gain = gain);
    } else if (ioType === IOType.STRIP) {
      return (this.strips[id].Gain = gain);
    }
  }

  getMuteState(ioType: IOType, id: number) {
    if (ioType === IOType.BUS) {
      return this.buses[id].Mute;
    } else if (ioType === IOType.STRIP) {
      return this.strips[id].Mute;
    }
  }

  setMuteState(ioType: IOType, id: number, state: boolean) {
    if (ioType === IOType.BUS) {
      return (this.buses[id].Mute = state);
    } else if (ioType === IOType.STRIP) {
      return (this.strips[id].Mute = state);
    }
  }

  getStripOutputBuses(stripId: number) {
    return this.strips[stripId].outputBuses;
  }

  setStripOutputBuses(stripId: number, outputs: boolean[]) {
    const buses = {
      A: Object.fromEntries(
        outputs
          .slice(0, this.control.availableBuses.physical)
          .map((e, i) => [i + 1, e]),
      ),
      B: Object.fromEntries(
        outputs
          .slice(this.control.availableBuses.virtual * -1)
          .map((e, i) => [i + 1, e]),
      ),
    } as OutputBuses;
    return (this.strips[stripId].outputBuses = buses);
  }
}
