import {
  Body,
  Controller,
  Get,
  HttpException,
  HttpStatus,
  Param,
  Patch,
} from '@nestjs/common';
import { AppService, IOType } from 'src/app.service';

@Controller('/bus/:busId')
export class BusController {
  constructor(private readonly appService: AppService) {}

  @Get('/gain')
  getGain(@Param('busId') busId: string) {
    return { gain: this.appService.getGain(IOType.BUS, +busId) };
  }
  @Patch('/gain')
  setGain(@Param('busId') busId: string, @Body() { gain }: { gain: number }) {
    if (gain === undefined)
      throw new HttpException(
        '"gain" parameter not specified',
        HttpStatus.BAD_REQUEST,
      );
    if (typeof gain !== 'number' || isNaN(gain))
      throw new HttpException(
        'Invalid "gain" parameter type. Must be number',
        HttpStatus.BAD_REQUEST,
      );
    return { gain: this.appService.setGain(IOType.BUS, +busId, gain) };
  }

  @Get('/mute')
  getMuteState(@Param('busId') busId: string) {
    return { muteState: this.appService.getMuteState(IOType.BUS, +busId) };
  }
  @Patch('/mute')
  setMuteState(
    @Param('busId') busId: string,
    @Body() { muteState }: { muteState: boolean },
  ) {
    if (muteState === undefined)
      throw new HttpException(
        '"muteState" parameter not specified',
        HttpStatus.BAD_REQUEST,
      );
    if (typeof muteState !== 'boolean')
      throw new HttpException(
        'Invalid "muteState" parameter type. Must be boolean',
        HttpStatus.BAD_REQUEST,
      );
    return {
      muteState: this.appService.setMuteState(IOType.BUS, +busId, muteState),
    };
  }
}
