import {
  Body,
  Controller,
  Get,
  Header,
  HostParam,
  HttpException,
  HttpStatus,
  Param,
  Patch,
  Post,
  Query,
} from '@nestjs/common';
import { AppService, IOType } from '../app.service';

@Controller('/strip/:stripId')
export class StripController {
  constructor(private readonly appService: AppService) {}

  @Get('/gain')
  getGain(@Param('stripId') stripId: string) {
    return { gain: this.appService.getGain(IOType.STRIP, +stripId) };
  }
  @Patch('/gain')
  setGain(
    @Param('stripId') stripId: string,
    @Body() { gain }: { gain: number },
  ) {
    if (gain === undefined)
      throw new HttpException(
        '"gain" parameter not specified',
        HttpStatus.BAD_REQUEST,
      );
    if (typeof gain !== 'number' || isNaN(gain))
      throw new HttpException(
        'Invalid "gain" parameter type. Must be number',
        HttpStatus.BAD_REQUEST,
      );
    return { gain: this.appService.setGain(IOType.STRIP, +stripId, gain) };
  }

  @Get('/mute')
  getMuteState(@Param('stripId') stripId: string) {
    return { muteState: this.appService.getMuteState(IOType.STRIP, +stripId) };
  }
  @Patch('/mute')
  setMuteState(
    @Param('stripId') stripId: string,
    @Body() { muteState }: { muteState: boolean },
  ) {
    if (muteState === undefined)
      throw new HttpException(
        '"muteState" parameter not specified',
        HttpStatus.BAD_REQUEST,
      );
    if (typeof muteState !== 'boolean')
      throw new HttpException(
        'Invalid "muteState" parameter type. Must be boolean',
        HttpStatus.BAD_REQUEST,
      );
    return {
      muteState: this.appService.setMuteState(
        IOType.STRIP,
        +stripId,
        muteState,
      ),
    };
  }

  @Get('/outputs')
  getOutputs(@Param('stripId') stripId: string) {
    return this.appService.getStripOutputBuses(+stripId);
  }
  @Patch('/outputs')
  setOutputs(
    @Param('stripId') stripId: string,
    @Body() { outputs }: { outputs: boolean[] },
  ) {
    if (outputs === undefined)
      throw new HttpException(
        '"outputs" parameter not specified',
        HttpStatus.BAD_REQUEST,
      );
    const busesAmount = this.appService.getAvailableIOs().buses.total;
    if (
      !Array.isArray(outputs) ||
      outputs.some((e) => typeof e !== 'boolean') ||
      outputs.length < busesAmount
    )
      throw new HttpException(
        `Invalid "outputs" parameter. Must be array with ${busesAmount} boolean(s)`,
        HttpStatus.BAD_REQUEST,
      );
    return this.appService.setStripOutputBuses(+stripId, outputs);
  }
}
